@@ -15,7 +15,7 @@
 
 #define MAJOR 0
 #define MINOR 4
-#define PATCH 9
+#define PATCH 10
 
 static char *NAME =  "libxml2:xmlDoc";
 static char *DESTRUCTOR = "destructor:xmlFreeDoc";
@@ -35,12 +35,12 @@ convert_tree(GumboOutput *output, Options *opts) {
     return doc;
 }
 
-static inline libxml_doc*
-parse_with_options(const char* buffer, size_t buffer_length, Options *opts) {
+static libxml_doc*
+parse_with_options(const char* buffer, size_t buffer_length, Options *opts, const GumboTag context, GumboNamespaceEnum context_namespace) {
     GumboOutput *output = NULL;
     libxml_doc* doc = NULL;
     Py_BEGIN_ALLOW_THREADS;
-    output = gumbo_parse_with_options(&(opts->gumbo_opts), buffer, buffer_length);
+    output = gumbo_parse_fragment(&(opts->gumbo_opts), buffer, buffer_length, context, context_namespace);
     Py_END_ALLOW_THREADS;
     if (output == NULL) PyErr_NoMemory();
     else {
@@ -76,18 +76,33 @@ parse(PyObject UNUSED *self, PyObject *args, PyObject *kwds) {
     Options opts = {0};
     opts.stack_size = 16 * 1024;
     PyObject *kd = Py_True, *mx = Py_False, *ne = Py_False, *sn = Py_True;
+    char *fragment_context = NULL; Py_ssize_t fragment_context_sz = 0;
     opts.gumbo_opts = kGumboDefaultOptions;
     opts.gumbo_opts.max_errors = 0;  // We discard errors since we are not reporting them anyway
+    GumboNamespaceEnum fragment_namespace = GUMBO_NAMESPACE_HTML;
 
-    static char *kwlist[] = {"data", "namespace_elements", "keep_doctype", "maybe_xhtml", "line_number_attr", "sanitize_names", "stack_size", NULL};
+    static char *kwlist[] = {"data", "namespace_elements", "keep_doctype", "maybe_xhtml", "line_number_attr", "sanitize_names", "stack_size", "fragment_context", "fragment_namespace", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s#|OOOzOI", kwlist, &buffer, &sz, &ne, &kd, &mx, &(opts.line_number_attr), &sn, &(opts.stack_size))) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s#|OOOzOIz#i", kwlist, &buffer, &sz, &ne, &kd, &mx, &(opts.line_number_attr), &sn, &(opts.stack_size), &fragment_context, &fragment_context_sz, &fragment_namespace)) return NULL;
     opts.namespace_elements = PyObject_IsTrue(ne);
     opts.keep_doctype = PyObject_IsTrue(kd);
     opts.sanitize_names = PyObject_IsTrue(sn);
     opts.gumbo_opts.use_xhtml_rules = PyObject_IsTrue(mx);
-
-    doc = parse_with_options(buffer, (size_t)sz, &opts);
+    GumboTag context = GUMBO_TAG_LAST;
+    if (fragment_context && fragment_context_sz > 0) {
+        context = gumbo_tagn_enum(fragment_context, fragment_context_sz);
+        if (context == GUMBO_TAG_UNKNOWN) {
+            PyErr_Format(PyExc_KeyError, "Unknown fragment_context tag name: %s", fragment_context);
+            return NULL;
+        }
+    }
+    if (fragment_namespace != GUMBO_NAMESPACE_HTML) {
+        // causes infinite loops in gumbo, enable the non html fragment context tests
+        // in html5lib_adapter.py to trigger
+        PyErr_SetString(PyExc_KeyError, "Fragment parsing with non-HTML namespaces is not supported");
+        return NULL;
+    }
+    doc = parse_with_options(buffer, (size_t)sz, &opts, context, fragment_namespace);
     if (!doc) return NULL;
     return encapsulate(doc);
 }
@@ -187,6 +202,9 @@ inithtml_parser(void) {
     if (PyModule_AddIntMacro(m, MAJOR) != 0) INITERROR;
     if (PyModule_AddIntMacro(m, MINOR) != 0) INITERROR;
     if (PyModule_AddIntMacro(m, PATCH) != 0) INITERROR;
+    if (PyModule_AddIntMacro(m, GUMBO_NAMESPACE_HTML) != 0) INITERROR;
+    if (PyModule_AddIntMacro(m, GUMBO_NAMESPACE_SVG) != 0) INITERROR;
+    if (PyModule_AddIntMacro(m, GUMBO_NAMESPACE_MATHML) != 0) INITERROR;
     if (PyModule_AddIntConstant(m, "LIBXML_VERSION", get_libxml_version()) != 0) INITERROR;
     known_tag_names = PyTuple_New(GUMBO_TAG_UNKNOWN);
     if (known_tag_names == NULL) INITERROR;
